@@ -18,7 +18,10 @@ use crate::remote::github::GitHubClient;
 use crate::remote::gitlab::GitLabClient;
 use crate::template::Template;
 use std::collections::HashMap;
-use std::io::Write;
+use std::io::{
+	Read,
+	Write,
+};
 use std::time::{
 	SystemTime,
 	UNIX_EPOCH,
@@ -39,8 +42,17 @@ pub struct Changelog<'a> {
 impl<'a> Changelog<'a> {
 	/// Constructs a new instance.
 	pub fn new(releases: Vec<Release<'a>>, config: &'a Config) -> Result<Self> {
+		let mut changelog = Changelog::build(releases, config)?;
+		changelog.add_remote_data()?;
+		changelog.process_commits();
+		changelog.process_releases();
+		Ok(changelog)
+	}
+
+	/// Builds a changelog from releases and config.
+	fn build(releases: Vec<Release<'a>>, config: &'a Config) -> Result<Self> {
 		let trim = config.changelog.trim.unwrap_or(true);
-		let mut changelog = Self {
+		Ok(Self {
 			releases,
 			header_template: match &config.changelog.header {
 				Some(header) => Some(Template::new(header.to_string(), trim)?),
@@ -61,11 +73,12 @@ impl<'a> Changelog<'a> {
 			},
 			config,
 			additional_context: HashMap::new(),
-		};
-		changelog.process_commits();
-		changelog.process_releases();
-		changelog.add_remote_data()?;
-		Ok(changelog)
+		})
+	}
+
+	/// Constructs an instance from a serialized context object.
+	pub fn from_context<R: Read>(input: &mut R, config: &'a Config) -> Result<Self> {
+		Changelog::build(serde_json::from_reader(input)?, config)
 	}
 
 	/// Adds a key value pair to the template context.
@@ -206,9 +219,9 @@ impl<'a> Changelog<'a> {
 	#[cfg(feature = "github")]
 	fn get_github_metadata(&self) -> Result<crate::remote::RemoteMetadata> {
 		use crate::remote::github;
-		if self
-			.body_template
-			.contains_variable(github::TEMPLATE_VARIABLES) ||
+		if self.config.remote.github.is_custom ||
+			self.body_template
+				.contains_variable(github::TEMPLATE_VARIABLES) ||
 			self.footer_template
 				.as_ref()
 				.map(|v| v.contains_variable(github::TEMPLATE_VARIABLES))
@@ -262,9 +275,9 @@ impl<'a> Changelog<'a> {
 	#[cfg(feature = "gitlab")]
 	fn get_gitlab_metadata(&self) -> Result<crate::remote::RemoteMetadata> {
 		use crate::remote::gitlab;
-		if self
-			.body_template
-			.contains_variable(gitlab::TEMPLATE_VARIABLES) ||
+		if self.config.remote.gitlab.is_custom ||
+			self.body_template
+				.contains_variable(gitlab::TEMPLATE_VARIABLES) ||
 			self.footer_template
 				.as_ref()
 				.map(|v| v.contains_variable(gitlab::TEMPLATE_VARIABLES))
@@ -326,9 +339,9 @@ impl<'a> Changelog<'a> {
 	#[cfg(feature = "gitea")]
 	fn get_gitea_metadata(&self) -> Result<crate::remote::RemoteMetadata> {
 		use crate::remote::gitea;
-		if self
-			.body_template
-			.contains_variable(gitea::TEMPLATE_VARIABLES) ||
+		if self.config.remote.gitea.is_custom ||
+			self.body_template
+				.contains_variable(gitea::TEMPLATE_VARIABLES) ||
 			self.footer_template
 				.as_ref()
 				.map(|v| v.contains_variable(gitea::TEMPLATE_VARIABLES))
@@ -379,9 +392,9 @@ impl<'a> Changelog<'a> {
 	#[cfg(feature = "bitbucket")]
 	fn get_bitbucket_metadata(&self) -> Result<crate::remote::RemoteMetadata> {
 		use crate::remote::bitbucket;
-		if self
-			.body_template
-			.contains_variable(bitbucket::TEMPLATE_VARIABLES) ||
+		if self.config.remote.bitbucket.is_custom ||
+			self.body_template
+				.contains_variable(bitbucket::TEMPLATE_VARIABLES) ||
 			self.footer_template
 				.as_ref()
 				.map(|v| v.contains_variable(bitbucket::TEMPLATE_VARIABLES))
@@ -499,7 +512,7 @@ impl<'a> Changelog<'a> {
 	}
 
 	/// Generates the changelog and writes it to the given output.
-	pub fn generate<W: Write>(&self, out: &mut W) -> Result<()> {
+	pub fn generate<W: Write + ?Sized>(&self, out: &mut W) -> Result<()> {
 		debug!("Generating changelog...");
 		let postprocessors = self
 			.config
@@ -567,7 +580,7 @@ impl<'a> Changelog<'a> {
 	}
 
 	/// Generates a changelog and prepends it to the given changelog.
-	pub fn prepend<W: Write>(
+	pub fn prepend<W: Write + ?Sized>(
 		&self,
 		mut changelog: String,
 		out: &mut W,
@@ -582,7 +595,7 @@ impl<'a> Changelog<'a> {
 	}
 
 	/// Prints the changelog context to the given output.
-	pub fn write_context<W: Write>(&self, out: &mut W) -> Result<()> {
+	pub fn write_context<W: Write + ?Sized>(&self, out: &mut W) -> Result<()> {
 		let output = Releases {
 			releases: &self.releases,
 		}
@@ -784,6 +797,7 @@ mod test {
 				tag_pattern:              None,
 				skip_tags:                Regex::new("v3.*").ok(),
 				ignore_tags:              None,
+				count_tags:               None,
 				topo_order:               Some(false),
 				sort_commits:             Some(String::from("oldest")),
 				link_parsers:             None,
@@ -791,24 +805,28 @@ mod test {
 			},
 			remote:    RemoteConfig {
 				github:    Remote {
-					owner: String::from("coolguy"),
-					repo:  String::from("awesome"),
-					token: None,
+					owner:     String::from("coolguy"),
+					repo:      String::from("awesome"),
+					token:     None,
+					is_custom: false,
 				},
 				gitlab:    Remote {
-					owner: String::from("coolguy"),
-					repo:  String::from("awesome"),
-					token: None,
+					owner:     String::from("coolguy"),
+					repo:      String::from("awesome"),
+					token:     None,
+					is_custom: false,
 				},
 				gitea:     Remote {
-					owner: String::from("coolguy"),
-					repo:  String::from("awesome"),
-					token: None,
+					owner:     String::from("coolguy"),
+					repo:      String::from("awesome"),
+					token:     None,
+					is_custom: false,
 				},
 				bitbucket: Remote {
-					owner: String::from("coolguy"),
-					repo:  String::from("awesome"),
-					token: None,
+					owner:     String::from("coolguy"),
+					repo:      String::from("awesome"),
+					token:     None,
+					is_custom: false,
 				},
 			},
 			bump:      Bump::default(),
@@ -816,6 +834,7 @@ mod test {
 		let test_release = Release {
 			version: Some(String::from("v1.0.0")),
 			message: None,
+			extra: None,
 			commits: vec![
 				Commit::new(
 					String::from("coffee"),
@@ -912,6 +931,7 @@ mod test {
 			Release {
 				version: None,
 				message: None,
+				extra: None,
 				commits: vec![
 					Commit::new(
 						String::from("abc123"),
