@@ -14,6 +14,9 @@ use std::fs;
 use std::path::Path;
 use std::path::PathBuf;
 
+/// Default initial tag.
+const DEFAULT_INITIAL_TAG: &str = "0.1.0";
+
 /// Manifest file information and regex for matching contents.
 #[derive(Debug)]
 struct ManifestInfo {
@@ -109,6 +112,9 @@ pub struct GitConfig {
 	/// Regex to ignore matched tags.
 	#[serde(with = "serde_regex", default)]
 	pub ignore_tags:              Option<Regex>,
+	/// Regex to count matched tags.
+	#[serde(with = "serde_regex", default)]
+	pub count_tags:               Option<Regex>,
 	/// Whether to sort tags topologically.
 	pub topo_order:               Option<bool>,
 	/// Sorting of the commits inside sections.
@@ -138,12 +144,20 @@ pub struct RemoteConfig {
 #[derive(Debug, Default, Clone, Serialize, Deserialize)]
 pub struct Remote {
 	/// Owner of the remote.
-	pub owner: String,
+	pub owner:     String,
 	/// Repository name.
-	pub repo:  String,
+	pub repo:      String,
 	/// Access token.
 	#[serde(skip_serializing)]
-	pub token: Option<SecretString>,
+	pub token:     Option<SecretString>,
+	/// Whether if the remote is set manually.
+	#[serde(skip_deserializing, default = "default_true")]
+	pub is_custom: bool,
+}
+
+/// Returns `true` for serde's `default` attribute.
+fn default_true() -> bool {
+	true
 }
 
 impl fmt::Display for Remote {
@@ -162,9 +176,10 @@ impl Remote {
 	/// Constructs a new instance.
 	pub fn new<S: Into<String>>(owner: S, repo: S) -> Self {
 		Self {
-			owner: owner.into(),
-			repo:  repo.into(),
-			token: None,
+			owner:     owner.into(),
+			repo:      repo.into(),
+			token:     None,
+			is_custom: false,
 		}
 	}
 
@@ -174,6 +189,17 @@ impl Remote {
 	}
 }
 
+/// Version bump type.
+#[derive(Debug, Clone, Copy, Serialize, Deserialize, Eq, PartialEq)]
+pub enum BumpType {
+	/// Bump major version.
+	Major,
+	/// Bump minor version.
+	Minor,
+	/// Bump patch version.
+	Patch,
+}
+
 /// Bump version configuration.
 #[derive(Debug, Default, Clone, Serialize, Deserialize)]
 pub struct Bump {
@@ -218,6 +244,33 @@ pub struct Bump {
 	///
 	/// `commit type` according to the spec is only `[a-zA-Z]+`
 	pub custom_minor_increment_regex: Option<String>,
+
+	/// Force to always bump in major, minor or patch.
+	pub bump_type: Option<BumpType>,
+}
+
+impl Bump {
+	/// Returns the initial tag.
+	///
+	/// This function also logs the returned value.
+	pub fn get_initial_tag(&self) -> String {
+		match self.initial_tag.clone() {
+			Some(tag) => {
+				warn!(
+					"No releases found, using initial tag '{tag}' as the next \
+					 version."
+				);
+				tag
+			}
+			None => {
+				warn!(
+					"No releases found, using {DEFAULT_INITIAL_TAG} as the next \
+					 version."
+				);
+				DEFAULT_INITIAL_TAG.into()
+			}
+		}
+	}
 }
 
 /// Parser for grouping commits.
