@@ -118,6 +118,7 @@ fn generate_changelog() -> Result<()> {
 		tag_pattern:              None,
 		skip_tags:                None,
 		ignore_tags:              None,
+		count_tags:               None,
 		topo_order:               None,
 		sort_commits:             None,
 		link_parsers:             Some(vec![
@@ -150,6 +151,7 @@ fn generate_changelog() -> Result<()> {
 		Release {
 			version:   Some(String::from("v2.0.0")),
 			message: None,
+            extra: None,
 			commits:   vec![
 
 				Commit::new(
@@ -215,6 +217,7 @@ fn generate_changelog() -> Result<()> {
 		Release {
 			version:   Some(String::from("v1.0.0")),
 			message: None,
+            extra: None,
 			commits:   vec![
 				Commit::new(
 					String::from("0bc123"),
